function [dx, duals] = form_solve_splp										...
				(alg, 										...
				baseMVA, gen, branch, bus, ref, pv, pq, nPPg,					...
				f_lp0, beta, Mppg, Emtrx, PPg, PPg_u, stepsize_P, stepsize_Q, norm_slackvar,	...
				dSbus_dVm, dSbus_dVa,								...
				dAf_dVa, dAf_dVm, dAt_dVa, dAt_dVm);

% FORM_SOLVE_SPLP formulates the lp sub-problem in opf. The lp problem is sparse based, that is, load flow
% equations are preserved.

%   MATPOWER Version 1.0
%   by Deqiang (David) Gan, PSERC Cornell    9/17/97
%   Copyright (c) 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

% ------ get ready to go ------

%% define named indices into bus, gen, branch matrices

[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
        VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
        RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;

%% set up constants and intial values (in quasi-alphabetical order)

on = find(gen(:, GEN_STATUS));                                                          %% which gens are on-line
gbus = gen(on, GEN_BUS);                                                                        %% what buses are they at?
j = sqrt (-1);                                                                                          %% operator of complex nu$
nb = size(bus, 1);                                                                                      %% number of buses
nb1= nb-1;                                                                                                      %% frequently use$
nl = size(branch, 1);                                                                           %% number of branches
npq     = length(pq);                                                                                   %% number of pq buses
npv     = length(pv);                                                                                   %% number of pv buses
ng = npv +1;                                                                                            %% number of on-line gens
pvpq = [1:ref-1, ref+1:size(bus, 1)];                                           %% indices of pv & pq buses

% formulate the LP

a_lp = [
	speye(ng, ng), sparse(ng, ng), -Mppg,	sparse(ng, nb),	 sparse(ng, nb1)		% PPg-Pg equations

	-Emtrx,			sparse(nb, ng), sparse(nb, nPPg), real(dSbus_dVm), real(dSbus_dVa(:, pvpq)) 	% P load flow equations
	sparse(nb, ng), -Emtrx,			sparse(nb, nPPg), imag(dSbus_dVm), imag(dSbus_dVa(:, pvpq))		% Q load flow equations
	
	sparse(nl, ng), sparse(nl, ng), sparse(nl, nPPg), dAf_dVm,		 dAf_dVa(:, pvpq)		% line flow inequalities
	sparse(nl, ng), sparse(nl, ng), sparse(nl, nPPg), dAt_dVm,		 dAt_dVa(:, pvpq)		% line flow inequalities
	
	sparse(nb, ng), sparse(nb, ng), sparse(nb, nPPg), speye(nb),	 sparse(nb, nb1)		% voltage inequalities
	sparse(nb, ng), sparse(nb, ng), sparse(nb, nPPg), -speye(nb), 	 sparse(nb, nb1)		% voltage inequalities

	speye(ng),		sparse(ng, ng), sparse(ng, nPPg), sparse(ng, nb),	sparse(ng, nb1)		% Pg limits
	-speye(ng),		sparse(ng, ng), sparse(ng, nPPg), sparse(ng, nb),	sparse(ng, nb1)		% Pg limits

	sparse(ng, ng), speye(ng), 		sparse(ng, nPPg), sparse(ng, nb), sparse(ng, nb1)		% Qg limits
	sparse(ng, ng), -speye(ng),		sparse(ng, nPPg), sparse(ng, nb), sparse(ng, nb1), 		% Qg limits
		];

rhs_lp=[
	zeros(ng, 1)

	zeros(nb, 1)
	zeros(nb, 1)
		
	branch(:, RATE_A)/baseMVA - (abs(branch(:, PF) + j * branch(:, QF)))/baseMVA				%% line flow limits
	branch(:, RATE_A)/baseMVA - (abs(branch(:, PT) + j * branch(:, QT)))/baseMVA				%% line flow limits
		
	bus(:, VMAX)			  - bus(:, VM)
	bus(:, VM)			- bus(:, VMIN)

	(gen(on, PMAX) 			  - gen(on, PG))/baseMVA
	(gen(on, PG) 			  - gen(on, PMIN))/baseMVA																
		
 	(gen(on, QMAX) 			- gen(on, QG))/baseMVA
 	(gen(on, QG) 			- gen(on, QMIN))/baseMVA
		];

f_lp = f_lp0;
constr_sense(1:ng+nb+nb) = zeros(ng+nb+nb, 1); 
constr_sense(ng+nb+nb+1:length(rhs_lp))=-1*ones(length(rhs_lp)-nb-nb-ng, 1);
num_of_equations = ng+1;

vub = [stepsize_P*ones(ng, 1); stepsize_Q*ones(ng, 1);  PPg_u - PPg; 4*ones(nb, 1);  4*ones(nb1, 1);];
vlb = [-stepsize_P*ones(ng,1); -stepsize_Q*ones(ng, 1);  - PPg; -4*ones(nb, 1); 	-4*ones(nb1, 1);]; 

%% add slack variable

if norm_slackvar > 1.0e-12
     nslack =2*nl+2*nb+4*ng;
     f_lp =[f_lp0; 80*baseMVA*max(beta)*ones(nslack, 1)];
     temp = [sparse(ng+2*nb, nslack);
             -speye(nslack)];
     a_lp = [a_lp, temp];
     vub = [vub; 100 * ones(nslack, 1)];
     vlb = [vlb; zeros(nslack, 1)];
end;



[dx, duals] = LPlpslvr(alg, a_lp, f_lp, rhs_lp, constr_sense, num_of_equations, vlb, vub);   

return;

